using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Globalization;
using System.Text;
using System.Web;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
using System.Reflection;

namespace XmlProviders.Data
{
    /// <summary>
    /// Contains a collection of <see cref="User" />
    /// </summary>
    [Serializable()]
    [XmlRoot("users", Namespace="http://schemas.swiftus.com/User.xsd")]
    public class UserCollection : Collection<User>
    {
        private object _lockObject = new object();

        /// <summary>
        /// Initializes a new instance of the <see cref="UserCollection"/> class.
        /// </summary>
        public UserCollection() : base() { }

        internal UserCollection(IList<User> array) : base(array) { }

        /// <summary>
        /// Loads the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns></returns>
        public static UserCollection Load(string fileName)
        {            
            if (File.Exists(fileName))
            {
                UserCollection users  = null;

                Microsoft.Xml.Serialization.GeneratedAssembly.UserCollectionSerializer s =
                    new Microsoft.Xml.Serialization.GeneratedAssembly.UserCollectionSerializer();

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.Schemas.Add(null, XmlReader.Create(Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(UserCollection), "User.xsd")));

                using (XmlReader rdr = XmlReader.Create(fileName, settings))
                {
                    if (s.CanDeserialize(rdr))
                    {
                        users = (UserCollection)s.Deserialize(rdr);
                    }
                }

                return users;
            }
            else
            {
                return new UserCollection();
            }
        }

        /// <summary>
        /// Saves the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public void Save(string fileName)
        {
            Microsoft.Xml.Serialization.GeneratedAssembly.UserCollectionSerializer s =
                new Microsoft.Xml.Serialization.GeneratedAssembly.UserCollectionSerializer();

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Encoding = System.Text.Encoding.UTF8;

            XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
            ns.Add(string.Empty, "http://schemas.swiftus.com/User.xsd");

            lock (_lockObject)
            {
                using (XmlWriter writer = XmlWriter.Create(fileName, settings))
                {
                    s.Serialize(writer, this, ns);
                }
            }
        }

        /// <summary>
        /// Finds the name of the by user.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <returns></returns>
        public User FindByUserName(string userName)
        {
            foreach (User u in this)
            {
                if (string.Compare(u.UserName, userName, true, CultureInfo.CurrentCulture) == 0)
                {
                    return u;
                }
            }

            return null;
        }

        /// <summary>
        /// Finds the by email.
        /// </summary>
        /// <param name="email">The email.</param>
        /// <returns></returns>
        public User FindByEmail(string email)
        {
            foreach (User u in this)
            {
                if (string.Compare(u.Email, email, true, CultureInfo.CurrentCulture) == 0)
                {
                    return u;
                }
            }
            return null;
        }

        /// <summary>
        /// Finds the by key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        public User FindByKey(Guid key)
        {
            foreach (User u in this)
            {
                if (u.Key.Equals(key))
                {
                    return u;
                }
            }
            return null;
        }

        /// <summary>
        /// Finds all active users.
        /// </summary>
        /// <returns></returns>
        public UserCollection FindAllActiveUsers()
        {
            UserCollection users = new UserCollection();

            foreach (User u in this)
            {
                if (u.IsActive) { users.Add(u); }
            }

            return users;
        }
    }
}
