using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace XmlProviders.Data
{
    /// <summary>
    /// Contains methods and properties to persist and retrieve user personalization data.
    /// </summary>
    [Serializable()]
    [XmlRoot("workspaces", Namespace = "http://schemas.swiftus.com/Workspace.xsd")]
    public class WorkspaceCollection
    {
        private object _lockObject = new object();
        private List<Workspace> _workspaces;
        private byte[] _shared;

        /// <summary>
        /// Gets or sets the shared personalization BLOB.
        /// </summary>
        /// <value>The shared personalization BLOB.</value>
        [XmlElement(typeof(byte[]), ElementName = "sharedPersonalization")]
        public byte[] SharedPersonalizationBlob
        {
            get { return _shared; }
            set { _shared = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkspaceCollection"/> class.
        /// </summary>
        public WorkspaceCollection() 
        {
            _workspaces = new List<Workspace>();
        }

        internal WorkspaceCollection(IList<Workspace> workspaces)
        {
            _workspaces = new List<Workspace>(workspaces);
        }

        /// <summary>
        /// Returns a user's workspace for a given index.
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Workspace this[int index]
        {
            get { return _workspaces[index]; }
            set { _workspaces[index] = value; }
        }

        /// <summary>
        /// Returns the collection of user workspaces.
        /// </summary>
        [XmlElement("Workspace", typeof(Workspace))]
        public Collection<Workspace> Workspaces
        {
            get { return new Collection<Workspace>(_workspaces); }
        }

        /// <summary>
        /// Loads the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns></returns>
        public static WorkspaceCollection Load(string fileName)
        {
            if (File.Exists(fileName))
            {
                WorkspaceCollection spaces = null;

                Microsoft.Xml.Serialization.GeneratedAssembly.WorkspaceCollectionSerializer s =
                    new Microsoft.Xml.Serialization.GeneratedAssembly.WorkspaceCollectionSerializer();

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.Schemas.Add(null, XmlReader.Create(Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(WorkspaceCollection), "Workspace.xsd")));

                using (XmlReader rdr = XmlReader.Create(fileName, settings))
                {
                    if (s.CanDeserialize(rdr))
                    {
                        spaces = (WorkspaceCollection)s.Deserialize(rdr);
                    }
                }

                return spaces;
            }
            else
            {
                return new WorkspaceCollection();
            }
        }

        /// <summary>
        /// Saves the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public void Save(string fileName)
        {
            Microsoft.Xml.Serialization.GeneratedAssembly.WorkspaceCollectionSerializer s =
                new Microsoft.Xml.Serialization.GeneratedAssembly.WorkspaceCollectionSerializer();

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Encoding = System.Text.Encoding.UTF8;

            XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
            ns.Add(string.Empty, "http://schemas.swiftus.com/Workspace.xsd");

            lock (_lockObject)
            {
                using (XmlWriter writer = XmlWriter.Create(fileName, settings))
                {
                    s.Serialize(writer, this, ns);
                }
            }
        }

        /// <summary>
        /// Finds the by user and path.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="path">The path.</param>
        /// <returns></returns>
        public Workspace FindByUserAndPath(string userName, string path)
        {
            foreach (Workspace w in _workspaces)
            {
                if (string.Compare(w.UserName, userName, true, CultureInfo.CurrentCulture) == 0)
                {
                    if (string.Compare(w.Path, path, true, CultureInfo.CurrentCulture) == 0) { return w; }
                }
            }

            return null;
        }

        /// <summary>
        /// Finds the by path.
        /// </summary>
        /// <param name="path">The path.</param>
        /// <returns></returns>
        public Collection<Workspace> FindByPath(string path)
        {
            Collection<Workspace> spaces = new Collection<Workspace>();

            foreach (Workspace w in _workspaces)
            {
                if (string.Compare(w.Path, path, true, CultureInfo.CurrentCulture) == 0)
                {
                    spaces.Add(w);
                }
            }

            return spaces;
        }

        /// <summary>
        /// Finds the name of the by user.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <returns></returns>
        public Collection<Workspace> FindByUserName(string userName)
        {
            Collection<Workspace> spaces = new Collection<Workspace>();

            foreach (Workspace w in _workspaces)
            {
                if (string.Compare(w.UserName, userName, true, CultureInfo.CurrentCulture) == 0)
                {
                    spaces.Add(w);
                }
            }

            return spaces;
        }
    }
}
